import re
from urllib.request import urlopen


class YtLive2m3u:
    # type: (str) -> str
    @staticmethod
    def meta_fields_to_extinf(line):
        """<channel name> | <group name> | <logo> | <tvg-id>"""
        fields = [i.strip() for i in line.split("|")]
        nf = len(fields)
        if nf != 4:
            raise ValueError("fields got {}, expected 4\nline: {}".format(nf, line))
        else:
            ch_name, grp_title, tvg_logo, tvg_id = fields
            return '#EXTINF:-1 group-title="{}" tvg-logo="{}" tvg-id="{}", {}'.format(
                grp_title.title(), tvg_logo, tvg_id, ch_name
            )

    # type: (str) -> str
    @classmethod
    def convert_ytlive_to_m3u(cls, url):
        """https://www.youtube.com/(?:user|channel)/[a-zA-Z0-9_-]+/live"""
        if not cls.is_valid_url(url):
            raise ValueError(url)
        response = urlopen(url, timeout=15).read().decode('utf-8')
        m = re.findall(r'https://[^"]+.m3u', response)
        if len(m) != 1:
            return "https://raw.githubusercontent.com/eggplants/YouTube_to_m3u/main/assets/moose_na.m3u"
        else:
            return m[0]

    # type: (str) -> bool
    @staticmethod
    def is_valid_url(url):
        test1 = re.match(r'^https://www\.youtube\.com/(?:user|channel)/[a-zA-Z0-9_-]+/live/?$', url)
        test2 = re.match(r'^https://www\.youtube\.com/watch\?v=[a-zA-Z0-9_-]+', url)
        test3 = re.match(r'^https://www\.youtube\.com/c/[a-zA-Z0-9_-]+/live/?$', url)
        return any((test1, test2, test3,))

    # type: (str) -> list[str]
    @classmethod
    def parse_info(cls, path):
        res = []
        is_url = False
        for line in open(path, "r").readlines():
            line = line.strip()
            if line == "" or line.startswith("~~"):
                continue
            elif not is_url:
                res.append(cls.meta_fields_to_extinf(line))
                is_url = True
            elif is_url and cls.is_valid_url(line):
                res.append(cls.convert_ytlive_to_m3u(line))
                is_url = False
            else:
                raise ValueError("info file is maybe invalid syntax\n{}".format(line))
        else:
            return res



import argparse

from y2m import __version__, y2m


# type: (str) -> str
def check_input_url(url):
    if y2m.is_valid_url(url):
        return url
    else:
        raise ValueError("{} is invalid url".format(url))


# type: (list[str] | None) -> argparse.Namespace
def parse_args(test=None):
    parser = argparse.ArgumentParser(
        description="Convert YouTube Live link into m3u one",
        epilog=(
            'valid url pattern:\n'
            '/^https://www.youtube.com/(?:user|channel)/[a-zA-Z0-9_-]+/live/?$/\n'
            '/^https://www.youtube.com/watch?v=[a-zA-Z0-9_-]+/\n'
            '/^https://www.youtube.com/c/[a-zA-Z0-9_-]+/live/?$/'
        )
    )

    parser.add_argument(
        "url",
        type=check_input_url,
        help='input YouTube url',
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version="%(prog)s {}".format(__version__),
    )
    if test is None:
        args = parser.parse_args(test)
    else:
        args = parser.parse_args()
    return args


# type: (list[str] | None) -> argparse.Namespace
def main(test=None) -> None:
    if test is None:
        args = parse_args(test)
    else:
        args = parse_args()
    print(y2m.convert_ytlive_to_m3u(args.url))


if __name__ == "__main__":
    main()



import argparse
from os.path import isdir, isfile

from y2m import __version__, y2m


# type: (str) -> str
def check_input_path(path):
    if isdir(path):
        raise ValueError("{} is dir".format(path))
    elif isfile(path):
        return path
    else:
        raise FileNotFoundError(path)


# type: (str) -> tuple[str, bool]
def check_output_path(path):
    if isdir(path):
        raise ValueError("{} is dir".format(path))
    elif isfile(path):
        return (path, True)
    else:
        return (path, False)


# type: (list[str] | None) -> argparse.Namespace
def parse_args(test=None):
    parser = argparse.ArgumentParser(
        description="Convert YouTube Live info file into m3u",
        epilog=(
            "example input file:\n"
            "https://git.io/JMQ7B"
            ))

    parser.add_argument(
        "info",
        type=check_input_path,
        help=(
            "input YouTube Live info file path"
        ),
    )
    parser.add_argument(
        "-o", "--out", type=check_output_path, help="output m3u path (overwrite: `-f`)"
    )
    parser.add_argument(
        "-f", "--force", action="store_true", help="overwrite if output path is exist"
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version="%(prog)s {}".format(__version__),
    )
    if test is None:
        args = parser.parse_args(test)
    else:
        args = parser.parse_args()
    return args


# type: (list[str] | None) -> argparse.Namespace
def main(test=None) -> None:
    if test is None:
        args = parse_args(test)
    else:
        args = parse_args()
    if args.out is None:
        res = y2m.parse_info(args.info)
        print("\n".join(res))
    elif args.out[0]:
        if (args.out[1] and args.force) or not args.out[1]:
            res = y2m.parse_info(args.info)
            print("\n".join(res), file=open(args.out[0], "w"))
            print("wrote:", args.out[0])
        else:
            raise ValueError("output path is already exist. use `-f` if overwrite")


if __name__ == "__main__":
    main()
